(*************************************************************************
 * UNRAR.DLL Demo                                                        *
 *                                                                       *
 * Copyright (c) 1993-96 by Eugene Roshal                                *
 * Port to TMT Pascal by Vadim Bodrov, TMT Development Corporation       *
 *************************************************************************)

// UNRAR.DLL is a 32-bit Windows dynamic-link library which provides file
// extraction from RAR archives. It handles archives created with RAR 2.0
// only and cannot extract files from archives of previous versions of RAR.

program UnRDLL;
uses CRT, Strings, Windows, UnRAR;

const
  EXTRACT = 1;
  TEST    = 2;
  PRINT   = 3;

function ChangeVolProc conv arg_cdecl (ArcName: PChar; Mode: Longint): LongBool;
var
  Ch: Char;
begin
  if Mode = RAR_VOL_ASK then
  begin
    Writeln('Insert disk with ', StrPas(ArcName), ' and press "Enter" or enter "Q" to exit');
    Ch := ReadKey;
    Result := LongBool(UpCase(Ch) <> 'Q');
  end else
    Result := 1;
end;

procedure OutOpenArchiveError(Error: Longint; ArcName: PChar);
begin
  case Error of
    ERAR_NO_MEMORY:   Writeln('Not enough memory');
    ERAR_EOPEN:       Writeln('Cannot open ', StrPas(ArcName));
    ERAR_BAD_ARCHIVE: Writeln(StrPas(ArcName)+' is not RAR archive');
    ERAR_BAD_DATA:    Writeln(Strpas(ArcName)+': archive header broken');
  end;
end;

procedure ShowComment(CmtBuf: PChar);
begin
  Writeln(#13+'Comment:'+13, StrPas(CmtBuf));
end;

function ProcessDataProc conv arg_cdecl (Addr: Pointer; Size: Longint): Longint;
var
  i: Longint;
begin
  Flush(Output);
  for i := 0 to Size - 1 do
    Write(Output, Char((Addr + i)^));
  Flush(Output);
  Result := 0;
end;

procedure OutProcessFileError(Error: Longint);
begin
  case Error of
    ERAR_UNKNOWN_FORMAT: Writeln('Unknown archive format');
    ERAR_BAD_ARCHIVE:    Writeln('Bad volume');
    ERAR_ECREATE:        Writeln('File create error');
    ERAR_EOPEN:          Writeln('Volume open error');
    ERAR_ECLOSE:         Writeln('File close error');
    ERAR_EREAD:          Writeln('Read error');
    ERAR_EWRITE:         Writeln('Write error');
    ERAR_BAD_DATA:       Writeln('CRC error');
  end;
end;

function LeadingSpaces(S: String): String;
begin
  if Length(S) < 24 then
    Result := S + ' ' * (24 - Length(S))
  else
    Result := S;
end;

procedure ExtractArchive(ArcName: PChar; Mode: Longint);
var
  hArcData: THandle;
  RHCode, PFCode: Longint;
  CmtBuf: array [0..16383] of Char;
  HeaderData: RARHeaderData;
  OpenArchiveData: RAROpenArchiveData;
begin
  OpenArchiveData.ArcName := ArcName;
  OpenArchiveData.CmtBuf := CmtBuf;
  OpenArchiveData.CmtBufSize := SizeOf(CmtBuf);
  OpenArchiveData.OpenMode := RAR_OM_EXTRACT;
  hArcData := RAROpenArchive(OpenArchiveData);

  if OpenArchiveData.OpenResult <> 0 then
  begin
    OutOpenArchiveError(OpenArchiveData.OpenResult,ArcName);
    exit;
  end;

  if OpenArchiveData.CmtState = 1 then
    ShowComment(CmtBuf);

  RARSetChangeVolProc(hArcData, @ChangeVolProc);

  if Mode = PRINT then
    RARSetProcessDataProc(hArcData, @ProcessDataProc);

  HeaderData.CmtBuf := NIL;

  RHCode := RARReadHeader(hArcData, HeaderData);

  while RHCode = 0 do
  begin
    case Mode of
      EXTRACT: Write('Extracting ', LeadingSpaces(StrPas(HeaderData.FileName)));
      TEST:    Write('Testing    ', LeadingSpaces(StrPas(HeaderData.FileName)));
      PRINT:   Write('Printing   ', LeadingSpaces(StrPas(HeaderData.FileName)));
    end;

    if Mode = EXTRACT then
      PFCode := RARProcessFile(hArcData, RAR_EXTRACT, NIL, NIL)
    else
      PFCode := RARProcessFile(hArcData, RAR_TEST, NIL, NIL);
    if PFCode = 0 then
      Writeln(' Ok')
    else begin
      OutProcessFileError(PFCode);
      Break;
    end;
    RHCode := RARReadHeader(hArcData, HeaderData);
  end;

  if RHCode = ERAR_BAD_DATA then
    Writeln('File header broken');

  RARCloseArchive(hArcData);
end;

procedure ListArchive(ArcName: PChar);
var
  hArcData: THandle;
  RHCode, PFCode: Longint;
  CmtBuf: array [0..16383] of CHar;
  HeaderData: RARHeaderData;
  OpenArchiveData: RAROpenArchiveData;
begin
  OpenArchiveData.ArcName := ArcName;
  OpenArchiveData.CmtBuf := CmtBuf;
  OpenArchiveData.CmtBufSize := SizeOf(CmtBuf);
  OpenArchiveData.OpenMode := RAR_OM_LIST;
  hArcData := RAROpenArchive(OpenArchiveData);

  if OpenArchiveData.OpenResult <> 0 then
  begin
    OutOpenArchiveError(OpenArchiveData.OpenResult, ArcName);
    exit;
  end;

  if OpenArchiveData.CmtState = 1 then
    ShowComment(CmtBuf);

  RARSetChangeVolProc(hArcData, @ChangeVolProc);

  HeaderData.CmtBuf := CmtBuf;
  HeaderData.CmtBufSize := SizeOf(CmtBuf);

  Writeln('File                    Size');
  Writeln('-------------------------------');
  RHCode := RARReadHeader(hArcData, HeaderData);
  while RHCode = 0 do
  begin
    Writeln(LeadingSpaces(StrPas(HeaderData.FileName)), HeaderData.UnpSize);
    if HeaderData.CmtState = 1 then
      ShowComment(CmtBuf);
    PFCode := RARProcessFile(hArcData, RAR_SKIP, NIL, NIL);
    if PFCode <> 0 then
    begin
      OutProcessFileError(PFCode);
      Break;
    end;
    RHCode := RARReadHeader(hArcData, HeaderData);
  end;

  if RHCode = ERAR_BAD_DATA then
    Writeln('File header broken');

  RARCloseArchive(hArcData);
end;

procedure OutHelp;
begin
  Writeln('UNRDLL. This is a simple example of UNRAR.DLL usage');
  Writeln('Syntax: UNRDLL X <Archive>     extract archive contents');
  Writeln('        UNRDLL T <Archive>     test archive contents');
  Writeln('        UNRDLL P <Archive>     print archive contents to stdout');
  Writeln('        UNRDLL L <Archive>     view archive contents');
end;

var
  TmpC: array [0..MAX_PATH] of Char;
begin
  if ParamCount <> 2 then
  begin
    OutHelp;
    Halt(0);
  end;
  Writeln;
  case UpCase(ParamStr(1)[1]) of
    'X': ExtractArchive(StrPCopy(TmpC, ParamStr(2)), EXTRACT);
    'T': ExtractArchive(StrPCopy(TmpC, ParamStr(2)), TEST);
    'P': ExtractArchive(StrPCopy(TmpC, ParamStr(2)), PRINT);
    'L': ListArchive(StrPCopy(TmpC, ParamStr(2)))
  else   OutHelp;
  end;
end.