(*************************************************************************
 * UNRAR Interface Unit (Win32)                                          *
 *                                                                       *
 * Copyright (c) 1993-96 by Eugene Roshal                                *
 * Port to TMT Pascal by Vadim Bodrov, TMT Development Corporation       *
 *************************************************************************)

// UNRAR.DLL is a 32-bit Windows dynamic-link library which provides file
// extraction from RAR archives. It handles archives created with RAR 2.0
// only and cannot extract files from archives of previous versions of RAR.

unit UnRAR;

interface

uses Windows;
{$w-}
const
  ERAR_END_ARCHIVE    = 10;
  ERAR_NO_MEMORY      = 11;
  ERAR_BAD_DATA       = 12;
  ERAR_BAD_ARCHIVE    = 13;
  ERAR_UNKNOWN_FORMAT = 14;
  ERAR_EOPEN          = 15;
  ERAR_ECREATE        = 16;
  ERAR_ECLOSE         = 17;
  ERAR_EREAD          = 18;
  ERAR_EWRITE         = 19;
  ERAR_SMALL_BUF      = 20;

  RAR_OM_LIST         = 0;
  RAR_OM_EXTRACT      = 1;

  RAR_SKIP            = 0;
  RAR_TEST            = 1;
  RAR_EXTRACT         = 2;

  RAR_VOL_ASK         = 0;
  RAR_VOL_NOTIFY      = 1;

type
  RARHeaderData = record
    ArcName: array [0..259] of Char;
    FileName: array [0..259] of Char;
    Flags: UINT;
    PackSize: UINT;
    UnpSize: UINT;
    HostOS: UINT;
    FileCRC: UINT;
    FileTime: UINT;
    UnpVer: UINT;
    Method: UINT;
    FileAttr: UINT;
    CmtBuf: PChar;
    CmtBufSize: UINT;
    CmtSize: UINT;
    CmtState: UINT;
end;

type
  RAROpenArchiveData = record
    ArcName: PChar;
    OpenMode: UINT;
    OpenResult: UINT;
    CmtBuf: PChar;
    CmtBufSize: UINT;
    CmtSize: UINT;
    CmtState: UINT;
end;

type
  TChangeVolProc   = ^function conv arg_cdecl (ArcName: PChar; Mode: Longint): Longint;
  TProcessDataProc = ^function conv arg_cdecl (Addr: Pointer; Size: Longint): Longint;

const
  unrardll = 'UNRAR.DLL';

function  RAROpenArchive conv arg_stdcall (ArchiveData: RAROpenArchiveData): THandle;
  external unrardll name 'RAROpenArchive';

function  RARCloseArchive conv arg_stdcall (hArcData: THandle): Longint;
  external unrardll name 'RARCloseArchive';

function RARReadHeader conv arg_stdcall (hArcData: THandle; HeaderData: RARHeaderData): Longint;
  external unrardll name 'RARReadHeader';

function  RARProcessFile conv arg_stdcall (hArcData: THandle; Operation: Longint; DestPath, DestName: PChar): Longint;
  external unrardll name 'RARProcessFile';

procedure RARSetChangeVolProc conv arg_stdcall (hArcData: THandle; ChangeVolProc: TChangeVolProc);
  external unrardll name 'RARSetChangeVolProc';

procedure RARSetProcessDataProc conv arg_stdcall (hArcData: THandle; ProcessDataProc: TProcessDataProc);
  external unrardll name 'RARSetProcessDataProc';

procedure RARSetPassword conv arg_stdcall (hArcData: Thandle; Password: PChar);
  external unrardll name 'RARSetPassword';

implementation

(* nothing to implement *)

end.